/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.0
  * @date     2020-11-02
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"

/* macro definition */
#define CRC32_TEST_1
//#define CRC32_TEST_2
//#define CRC32_TEST_3
//#define CRC32_TEST_4
//#define CRC32_TEST_5
//#define CRC16_TEST_1
//#define CRC16_TEST_2
//#define CRC16_TEST_3
//#define CRC16_TEST_4
//#define CRC16_TEST_5
//#define CRC8_TEST_1
//#define CRC8_TEST_2
//#define CRC8_TEST_3
//#define CRC8_TEST_4
//#define CRC8_TEST_5
//#define CRC16_TEST_8BIT_DATA
//#define CRC32_TEST_8BIT_DATA
//#define CRC32_TEST_16BIT_DATA

/* test buffer */
uint32_t data_buffer_32[] = {0x11223344, 0x55667788};
uint16_t data_buffer_16[] = {0x1122, 0x3344, 0x5566};
uint8_t  data_buffer_8[]  = {0x11, 0x22, 0x33, 0x44, 0x55};

/* crc value */
__IO uint32_t crc_value = 0;

/**
  * @brief  calculate 8-bit data CRC.
  * @param  pbuffer: pointer to the buffer containing the data to be computed.
  * @param  length: length of the buffer to be computed.
  * @retval crc
  */
uint32_t crc8_block_calculate(uint8_t *pbuffer, uint32_t length)
{
  uint32_t index = 0;

  for(index = 0; index < length; index++)
  {
    (*(uint8_t *)&CRC->dt) = pbuffer[index];
  }

  return (CRC->dt);
}

/**
  * @brief  calculate 16-bit data CRC.
  * @param  pbuffer: pointer to the buffer containing the data to be computed.
  * @param  length: length of the buffer to be computed.
  * @retval crc
  */
uint32_t crc16_block_calculate(uint16_t *pbuffer, uint32_t length)
{
  uint32_t index = 0;

  for(index = 0; index < length; index++)
  {
    (*(uint16_t *)&CRC->dt) = pbuffer[index];
  }

  return (CRC->dt);
}

/**
  * @brief  configure 32-bit CRC calculation.
  * @param  none.
  * @retval none.
  */
void crc32_config(void)
{
#if defined (CRC32_TEST_1 )
  /* config initial data */
  crc_init_data_set(0xFFFFFFFF);

  /* config poly default value 0x4C11DB7 */
  crc_poly_value_set(0x4C11DB7);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
#elif defined (CRC32_TEST_2 )
  
  /* config initial data */
  crc_init_data_set(0x00000000);

  /* config poly default value 0x4C11DB7 */
  crc_poly_value_set(0x4C11DB7);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
#elif defined (CRC32_TEST_3 )

  /* config initial data */
  crc_init_data_set(0xFFFFFFFF);

  /* config poly default value 0x4C11DB7 */
  crc_poly_value_set(0x4C11DB7);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
#elif defined (CRC32_TEST_4 )

  /* config initial data */
  crc_init_data_set(0x00000000);

  /* config poly default value 0x4C11DB7 */
  crc_poly_value_set(0x4C11DB7);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
#elif defined (CRC32_TEST_5 )

  /* config initial data */
  crc_init_data_set(0xFFFFFFFF);

  /* config poly default value 0xA833982B */
  crc_poly_value_set(0xA833982B);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
#endif
}

/**
  * @brief  configure 16-bit CRC calculation.
  * @param  none.
  * @retval none.
  */
void crc16_config(void)
{
#if defined (CRC16_TEST_1 )
  
  /* config initial data */
  crc_init_data_set(0xFFFF);

  /* config poly default value 0x8005 */
  crc_poly_value_set(0x8005);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
#elif defined (CRC16_TEST_2 )
  
  /* config initial data */
  crc_init_data_set(0x0000);

  /* config poly default value 0x8005 */
  crc_poly_value_set(0x8005);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
#elif defined (CRC16_TEST_3 )

  /* config initial data */
  crc_init_data_set(0xFFFF);

  /* config poly default value 0x8005 */
  crc_poly_value_set(0x8005);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
#elif defined (CRC16_TEST_4 )

  /* config initial data */
  crc_init_data_set(0x0000);

  /* config poly default value 0x8005 */
  crc_poly_value_set(0x8005);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
#elif defined (CRC16_TEST_5 )

  /* config initial data */
  crc_init_data_set(0xFFFF);

  /* config poly default value 0x3D65 */
  crc_poly_value_set(0x3D65);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
#endif
}

/**
  * @brief  configure 8-bit CRC calculation.
  * @param  none.
  * @retval none.
  */
void crc8_config(void)
{
#if defined (CRC8_TEST_1 )
  
  /* config initial data */
  crc_init_data_set(0xFF);

  /* config poly default value 0x31 */
  crc_poly_value_set(0x31);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
#elif defined (CRC8_TEST_2 )
  
  /* config initial data */
  crc_init_data_set(0x00);

  /* config poly default value 0x31 */
  crc_poly_value_set(0x31);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
#elif defined (CRC8_TEST_3 )

  /* config initial data */
  crc_init_data_set(0xFF);

  /* config poly default value 0x31 */
  crc_poly_value_set(0x31);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
#elif defined (CRC8_TEST_4 )

  /* config initial data */
  crc_init_data_set(0x00);

  /* config poly default value 0x31 */
  crc_poly_value_set(0x31);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
#elif defined (CRC8_TEST_5 )

  /* config initial data */
  crc_init_data_set(0xFF);

  /* config poly default value 0x07 */
  crc_poly_value_set(0x07);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
#endif
}

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  system_clock_config();
  
  at32_board_init();
  
  uart_print_init(115200);
  
  /* enable crc clock */
  crm_periph_clock_enable(CRM_CRC_PERIPH_CLOCK, TRUE);

#if defined (CRC32_TEST_1 )   
  printf ("CRC32_TEST_1\r\n"); 
#elif defined (CRC32_TEST_2 ) 
  printf ("CRC32_TEST_2\r\n"); 
#elif defined (CRC32_TEST_3 ) 
  printf ("CRC32_TEST_3\r\n"); 
#elif defined (CRC32_TEST_4 ) 
  printf ("CRC32_TEST_4\r\n"); 
#elif defined (CRC32_TEST_5 ) 
  printf ("CRC32_TEST_5\r\n"); 
#elif defined (CRC16_TEST_1 )   
  printf ("CRC16_TEST_1\r\n"); 
#elif defined (CRC16_TEST_2 ) 
  printf ("CRC16_TEST_2\r\n"); 
#elif defined (CRC16_TEST_3 ) 
  printf ("CRC16_TEST_3\r\n"); 
#elif defined (CRC16_TEST_4 ) 
  printf ("CRC16_TEST_4\r\n"); 
#elif defined (CRC16_TEST_5 ) 
  printf ("CRC16_TEST_5\r\n"); 
#elif defined (CRC8_TEST_1 )   
  printf ("CRC8_TEST_1\r\n"); 
#elif defined (CRC8_TEST_2 ) 
  printf ("CRC8_TEST_2\r\n"); 
#elif defined (CRC8_TEST_3 ) 
  printf ("CRC8_TEST_3\r\n"); 
#elif defined (CRC8_TEST_4 ) 
  printf ("CRC8_TEST_4\r\n"); 
#elif defined (CRC8_TEST_5 ) 
  printf ("CRC8_TEST_5\r\n"); 
#elif defined (CRC16_TEST_8BIT_DATA ) 
  printf ("CRC16_TEST_8BIT_DATA\r\n"); 
#elif defined (CRC32_TEST_8BIT_DATA ) 
  printf ("CRC32_TEST_8BIT_DATA\r\n");
#elif defined (CRC32_TEST_16BIT_DATA ) 
  printf ("CRC32_TEST_16BIT_DATA\r\n");
#endif
  
#if defined (CRC32_TEST_1 ) || \
    defined (CRC32_TEST_2 ) || \
    defined (CRC32_TEST_3 ) || \
    defined (CRC32_TEST_4 ) || \
    defined (CRC32_TEST_5 )
    
  /* config poly size 32-bit */
  crc_poly_size_set(CRC_POLY_SIZE_32B);
  
  crc32_config();
  
  /* reset CRC_DT */
  crc_data_reset();

  /* compute the crc of "data_buffer" */
  crc_value = crc_block_calculate((uint32_t *)data_buffer_32, 2);

  printf ("CRC Value:0x%08X\r\n", crc_value);    
    
#endif
  
#if defined (CRC16_TEST_1 ) || \
    defined (CRC16_TEST_2 ) || \
    defined (CRC16_TEST_3 ) || \
    defined (CRC16_TEST_4 ) || \
    defined (CRC16_TEST_5 )
    
  /* config poly size 16-bit */
  crc_poly_size_set(CRC_POLY_SIZE_16B);
  
  crc16_config();
  
  /* reset CRC_DT */
  crc_data_reset();

  /* compute the crc of "data_buffer" */
  crc_value = crc16_block_calculate((uint16_t *)data_buffer_16, 2);

  printf ("CRC Value:0x%04X\r\n", crc_value);    
    
#endif
  
#if defined (CRC8_TEST_1 ) || \
    defined (CRC8_TEST_2 ) || \
    defined (CRC8_TEST_3 ) || \
    defined (CRC8_TEST_4 ) || \
    defined (CRC8_TEST_5 ) 
    
  /* config poly size 8-bit */
  crc_poly_size_set(CRC_POLY_SIZE_8B);
  
  crc8_config();
  
  /* reset CRC_DT */
  crc_data_reset();

  /* compute the crc of "data_buffer" */
  crc_value = crc8_block_calculate((uint8_t *)data_buffer_8, 2);

  printf ("CRC Value:0x%02X\r\n", crc_value);    
#endif
  
#if defined (CRC16_TEST_8BIT_DATA ) 
  /* config poly size 16-bit */
  crc_poly_size_set(CRC_POLY_SIZE_16B);
  
  /* config initial data */
  crc_init_data_set(0xFFFF);

  /* config poly default value 0x8005 */
  crc_poly_value_set(0x8005);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_DATA);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_BY_BYTE);
  
  /* reset CRC_DT */
  crc_data_reset();

  /* compute the crc of "data_buffer" */
  crc_value = crc8_block_calculate((uint8_t *)data_buffer_8, 5);

  printf ("CRC Value:0x%04X\r\n", crc_value);    
#endif
  
#if defined (CRC32_TEST_8BIT_DATA ) 

  /* config poly size 32-bit */
  crc_poly_size_set(CRC_POLY_SIZE_32B);
  
  /* config initial data */
  crc_init_data_set(0xFFFFFFFF);

  /* config poly default value 0x4C11DB7 */
  crc_poly_value_set(0x4C11DB7);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
  /* reset CRC_DT */
  crc_data_reset();

  /* compute the crc of "data_buffer" */
  crc_value = crc8_block_calculate((uint8_t *)data_buffer_8, 5);

  printf ("CRC Value:0x%08X\r\n", crc_value);    
#endif
  
#if defined (CRC32_TEST_16BIT_DATA ) 

  /* config poly size 32-bit */
  crc_poly_size_set(CRC_POLY_SIZE_32B);
  
  /* config initial data */
  crc_init_data_set(0xFFFFFFFF);

  /* config poly default value 0x4C11DB7 */
  crc_poly_value_set(0x4C11DB7);

  /* config output data reverse mode */
  crc_reverse_output_data_set(CRC_REVERSE_OUTPUT_NO_AFFECTE);

  /* config input data reverse mode */
  crc_reverse_input_data_set(CRC_REVERSE_INPUT_NO_AFFECTE);
  
  /* reset CRC_DT */
  crc_data_reset();

  /* compute the crc of "data_buffer" */
  crc_value = crc16_block_calculate((uint16_t *)data_buffer_16, 3);

  printf ("CRC Value:0x%08X\r\n", crc_value);    
#endif

  while(1)
  {
  }
}
